package logic

/*
	desc: "AI core微服务"
	author: "xie"
*/

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/tool"
	"bytes"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"github.com/zeromicro/go-zero/core/logx"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"
	"strings"
)

import (
	"context"
)

type CreateServiceLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewCreateServiceLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreateServiceLogic {
	return &CreateServiceLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// service management
func (l *CreateServiceLogic) CreateService(in *modelarts.CreateServiceReq) (*modelarts.CreateServiceResp, error) {
	// todo: add your logic here and delete this line
	var resp modelarts.CreateServiceResp
	//根据智算类型判断走华为智算还是南京智算
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		url := modelArtsUrl + "v1/" + in.ProjectId + "/services"
		reqByte, err := json.Marshal(in)
		if err != nil {
			return nil, err
		}
		payload := strings.NewReader(string(reqByte))
		token := common.GetToken()
		statusCode, body, err := tool.HttpClientWithBodyAndCode(tool.POST, url, payload, token)
		if err != nil {
			return nil, err
		}
		if statusCode == 200 {
			json.Unmarshal(body, &resp)
			resp.Code = 200
			resp.Msg = "Success"
		} else if statusCode != 200 {
			json.Unmarshal(body, &resp)
			resp.Code = 400
			resp.Msg = "Failure"
		}
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		reqByte, err := json.Marshal(in)
		r, err := http.NewRequest("POST", NanjingModelArtsUrl+"v1/"+in.ProjectId+"/services",
			bytes.NewBuffer(reqByte))
		if err != nil {
			fmt.Println(err)
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}

	return &resp, nil
}
