package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/adaptor/PCM-CORE/rpc/pcmcoreclient"
	"context"
	"github.com/zeromicro/go-zero/core/logx"
	"time"
)

func InitCron(svc *svc.ServiceContext) {
	submitJobLogic := NewCreateTrainingJobLogic(context.Background(), svc)
	listLogic := NewGetListTrainingJobsLogic(context.Background(), svc)
	svc.Cron.AddFunc("*/5  *  *  *  *  ?", func() {
		syncInfoReq := pcmcoreclient.SyncInfoReq{
			Kind:        "ai",
			ServiceName: "modelArts",
		}
		// 查询core端分发下来的任务列表
		infoList, err := queryCoreInfoList(svc)
		if err != nil {
			logx.Error(err)
			return
		}
		// 提交任务
		submitJob(infoList, submitJobLogic)
		// 查询运行中的任务列表同步信息 吧吧你 你
		listReq := modelarts.ListTrainingJobsreq{
			ProjectId:     "0a62ffb0d48026c12fbfc011b8d23f0b",
			Limit:         10,
			OffSet:        0,
			ModelArtsType: "cn-north-4.myhuawei",
		}
		listJob, err := listLogic.GetListTrainingJobs(&listReq)
		if err != nil {
			logx.Error(err)
			return
		}
		for index, _ := range infoList.AiInfoList {
			for _, job := range listJob.Items {
				if job.Metadata.Name == infoList.AiInfoList[index].Name {
					infoList.AiInfoList[index].ProjectId = job.ProjectId
					infoList.AiInfoList[index].JobId = job.Metadata.Id
					createTime := time.Unix(int64(job.Metadata.CreateTime)/1000, 0)
					infoList.AiInfoList[index].CreateTime = time.Time.String(createTime)
					if job.Status.StartTime != 0 {
						startTime := time.Unix(int64(job.Status.StartTime)/1000, 0)
						infoList.AiInfoList[index].StartTime = time.Time.String(startTime)
					}
					infoList.AiInfoList[index].RunningTime = int64(job.Status.Duration) / 1000
					infoList.AiInfoList[index].Status = job.Status.Phase

				}
			}
		}
		// 同步信息到core端
		if len(infoList.AiInfoList) != 0 {
			syncInfoReq.AiInfoList = infoList.AiInfoList
			svc.PcmCoreRpc.SyncInfo(context.Background(), &syncInfoReq)
		}
	})
}

func submitJob(infoList *pcmcoreclient.InfoListResp, submitJobLogic *CreateTrainingJobLogic) {
	for index, _ := range infoList.AiInfoList {
		if infoList.AiInfoList[index].Status == "Saved" {
			submitReq := modelarts.CreateTrainingJobReq{
				ModelArtsType: "cn-north-4.myhuawei",
				Kind:          "job",
				ProjectId:     "0a62ffb0d48026c12fbfc011b8d23f0b",
				Metadata: &modelarts.MetadataS{
					Name:        infoList.AiInfoList[index].Name,
					WorkspaceId: "0",
					Description: "This is a ModelArts Demo Job",
				},
				Algorithm: &modelarts.Algorithms{
					Command: infoList.AiInfoList[index].Command,
					Engine: &modelarts.EngineCreateTraining{
						ImageUrl: infoList.AiInfoList[index].ImageUrl,
					},
				},
				Spec: &modelarts.SpecsC{
					Resource: &modelarts.ResourceCreateTraining{
						FlavorId:  infoList.AiInfoList[index].FlavorId,
						NodeCount: 1,
					},
				},
			}
			jobResult, _ := submitJobLogic.CreateTrainingJob(&submitReq)
			if jobResult.Code == 200 {
				infoList.AiInfoList[index].Status = jobResult.Status.Phase
				infoList.AiInfoList[index].ProjectId = jobResult.Metadata.Id
			} else {
				infoList.AiInfoList[index].Result = "Failed"
				infoList.AiInfoList[index].Result = jobResult.Msg
			}
		}
	}
}

func queryCoreInfoList(svc *svc.ServiceContext) (*pcmcoreclient.InfoListResp, error) {
	infoReq := pcmcoreclient.InfoListReq{
		Kind:        "ai",
		ServiceName: "modelArts",
	}
	infoList, err := svc.PcmCoreRpc.InfoList(context.Background(), &infoReq)
	if err != nil {
		return nil, err
	}
	return infoList, nil
}
