package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/tool"
	"context"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"github.com/zeromicro/go-zero/core/logx"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"
	"strings"
)

type DeleteTrainingJobLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDeleteTrainingJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DeleteTrainingJobLogic {
	return &DeleteTrainingJobLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// DeleteTrainingJobConfig 删除训练作业
func (l *DeleteTrainingJobLogic) DeleteTrainingJob(in *modelarts.DeleteTrainingJobReq) (*modelarts.DeleteTrainingJobResp, error) {
	var resp modelarts.DeleteTrainingJobResp
	//根据智算类型判断走华为智算还是南京智算
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		url := modelArtsUrl + "v2/" + in.ProjectId + "/training-jobs/" + in.TrainingJobId
		reqByte, err := json.Marshal(in)
		if err != nil {
			return nil, err
		}
		payload := strings.NewReader(string(reqByte))
		token := common.GetToken()
		statusCode, body, err := tool.HttpClientWithBodyAndCode(tool.DELETE, url, payload, token)
		if err != nil {
			return nil, err
		}
		if statusCode == 202 {
			json.Unmarshal(body, &resp)
			resp.Code = 200
			resp.Msg = "Success"
		} else if statusCode != 202 {
			json.Unmarshal(body, &resp)
			resp.Code = 400
			resp.Msg = "Failure"
		}
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		XDomainId := l.svcCtx.Config.XDomainId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("DELETE", NanjingModelArtsUrl+"v2/"+in.ProjectId+"/training-jobs/"+in.TrainingJobId,
			nil)
		if err != nil {
			fmt.Println(err)
			//return
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("X-Domain-Id", XDomainId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		statusCode := res.StatusCode
		if statusCode == 202 {
			json.Unmarshal(body, &resp)
			resp.Code = 200
			resp.Msg = "Success"
		} else if statusCode != 202 {
			json.Unmarshal(body, &resp)
			resp.Code = 400
			resp.Msg = "Failure"
		}
	}

	return &resp, nil
}
