package logic

/*
	desc: "AI core微服务"
	author: "xie"
*/

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/tool"
	"context"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"

	"github.com/zeromicro/go-zero/core/logx"
)

type DescribeProcessorTaskLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDescribeProcessorTaskLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DescribeProcessorTaskLogic {
	return &DescribeProcessorTaskLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *DescribeProcessorTaskLogic) DescribeProcessorTask(in *modelarts.DescribeProcessorTaskReq) (*modelarts.DescribeProcessorTaskResp, error) {
	// todo: add your logic here and delete this line
	var resp modelarts.DescribeProcessorTaskResp
	//根据智算类型判断走华为智算还是南京智算
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		url := modelArtsUrl + "v2/" + in.ProjectId + "/processor-tasks/" + in.TaskId
		token := common.GetToken()
		body, err := tool.HttpClient(tool.GET, url, nil, token)
		if err != nil {
			return nil, err
		}
		json.Unmarshal(body, &resp)
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("GET", NanjingModelArtsUrl+"v2/"+in.ProjectId+"/processor-tasks/"+in.TaskId,
			nil)
		if err != nil {
			fmt.Println(err)
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}
	return &resp, nil
}
