package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/tool"
	"context"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"github.com/zeromicro/go-zero/core/logx"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"
	"strconv"
	"strings"
)

type GetDatasetListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetDatasetListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetDatasetListLogic {
	return &GetDatasetListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// find datasetList
func (l *GetDatasetListLogic) GetDatasetList(in *modelarts.DataSetReq) (*modelarts.DataSetResp, error) {
	projectId := in.ProjectId
	var resp modelarts.DataSetResp
	offset := strconv.Itoa(int(in.Offset))
	judgeLimit := strconv.Itoa(int(in.Limit))
	var limit string
	if judgeLimit != "0" {
		limit = strconv.Itoa(int(in.Limit))
	} else {
		limit = "10"
	}
	//根据智算类型判断走华为智算还是南京智算
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		token := common.GetToken()
		statusCode, body, err := tool.HttpClientWithBodyAndCode(tool.GET, modelArtsUrl+"v2/"+projectId+"/datasets?offset="+offset+"&"+"limit="+limit, strings.NewReader(``), token)
		if err != nil {
			return nil, err
		}
		if statusCode == 200 {
			json.Unmarshal(body, &resp)
			resp.Code = 200
			resp.Msg = "Success"
		} else if statusCode != 200 {
			json.Unmarshal(body, &resp)
			resp.Code = 400
			resp.Msg = "Failure"
		}
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("GET", NanjingModelArtsUrl+"v2/"+in.ProjectId+"/datasets?offset="+offset+"&"+"limit="+limit,
			nil)
		if err != nil {
			fmt.Println(err)
			//return
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}

	return &resp, nil
}
