package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/common/tool"
	"context"
	"encoding/json"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"io/ioutil"
	"net/http"
	"strings"

	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetNotebookStorageLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetNotebookStorageLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetNotebookStorageLogic {
	return &GetNotebookStorageLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetNotebookStorageLogic) GetNotebookStorage(in *modelarts.GetNotebookStorageReq) (*modelarts.GetNotebookStorageResp, error) {
	var resp modelarts.GetNotebookStorageResp
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		getObsUrl := modelArtsUrl + "v1/{project_id}/notebooks/{instance_id}/storage"
		getObsUrl = strings.Replace(getObsUrl, "{project_id}", in.ProjectId, -1)
		getObsUrl = strings.Replace(getObsUrl, "{instance_id}", in.InstanceId, -1)
		token := common.GetToken()
		var e struct{}

		body, err := tool.HttpClientWithQueries(tool.GET, getObsUrl, nil, token, e)
		if err != nil {
			return nil, err
		}
		json.Unmarshal(body, &resp)
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("GET", NanjingModelArtsUrl+"v1/"+in.ProjectId+"/notebooks"+in.InstanceId+"/storage",
			nil)
		if err != nil {
			fmt.Println(err)
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}

	return &resp, nil
}
