package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/common/tool"
	"context"
	"encoding/json"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"io/ioutil"
	"net/http"
	"strings"

	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetVisualizationJobLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetVisualizationJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetVisualizationJobLogic {
	return &GetVisualizationJobLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// visualization-jobs
func (l *GetVisualizationJobLogic) GetVisualizationJob(in *modelarts.GetVisualizationJobReq) (*modelarts.GetVisualizationJobResp, error) {
	var resp modelarts.GetVisualizationJobResp
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		getVisualJobUrl := modelArtsUrl + "v1/{project_id}/visualization-jobs"
		getVisualJobUrl = strings.Replace(getVisualJobUrl, "{project_id}", in.ProjectId, -1)

		token := common.GetToken()

		body, err := tool.HttpClientWithQueries(tool.GET, getVisualJobUrl, nil, token, in.Param)
		if err != nil {
			return nil, err
		}
		json.Unmarshal(body, &resp)
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("GET", NanjingModelArtsUrl+"v1/"+in.ProjectId+"/visualization-jobs",
			nil)
		if err != nil {
			fmt.Println(err)
			//return
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}

	return &resp, nil
}
