package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/tool"
	"context"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"github.com/zeromicro/go-zero/core/logx"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"
)

type ListNotebookLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewListNotebookLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListNotebookLogic {
	return &ListNotebookLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// notebook task
func (l *ListNotebookLogic) ListNotebook(in *modelarts.ListNotebookReq) (*modelarts.ListNotebookResp, error) {
	resp := &modelarts.ListNotebookResp{}
	//根据智算类型判断走华为智算还是南京智算
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		getUrl := modelArtsUrl + "v1/{project_id}/notebooks"

		token := common.GetToken()
		queryMap := tool.ConvertStructToMap(in.Param)

		req := tool.GetACHttpRequest()
		res, err := req.
			SetHeader("x-auth-token", token).
			SetPathParam("project_id", in.ProjectId).
			SetQueryParams(queryMap).
			SetResult(resp).
			Get(getUrl)

		if err != nil {
			return nil, err
		}

		if res.StatusCode() != 200 {
			resp.Code = int32(res.StatusCode())
			resp.Msg = "Failure"

			var errMsg common.Error
			err := json.Unmarshal(res.Body(), &errMsg)
			if err != nil {
				errMsg.ErrorMsg = ""
			}
			resp.ErrorMsg = errMsg.ErrorMsg
		} else {
			resp.Code = int32(res.StatusCode())
			resp.Msg = "Success"
		}
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		r, err := http.NewRequest("GET", NanjingModelArtsUrl+"v1/"+in.ProjectId+"/notebooks",
			nil)
		if err != nil {
			fmt.Println(err)
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}

	return resp, nil
}
