package logic

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/common"
	"PCM/common/tool"
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"github.com/JCCE-nudt/apigw-go-sdk/core"
	"io/ioutil"
	"net/http"
	"strings"

	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"

	"github.com/zeromicro/go-zero/core/logx"
)

type StopNotebookLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewStopNotebookLogic(ctx context.Context, svcCtx *svc.ServiceContext) *StopNotebookLogic {
	return &StopNotebookLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *StopNotebookLogic) StopNotebook(in *modelarts.StopNotebookReq) (*modelarts.StopNotebookResp, error) {
	var resp modelarts.StopNotebookResp
	modelArtsType := in.ModelArtsType
	if modelArtsType == l.svcCtx.Config.HaweiModelArtsType {
		modelArtsUrl := l.svcCtx.Config.ModelArtsUrl
		stopUrl := modelArtsUrl + "v1/{project_id}/notebooks/{id}/stop"
		stopUrl = strings.Replace(stopUrl, "{project_id}", in.ProjectId, -1)
		stopUrl = strings.Replace(stopUrl, "{id}", in.Id, -1)
		token := common.GetToken()
		//empty struct
		var e struct{}
		body, err := tool.HttpClientWithQueries(tool.POST, stopUrl, nil, token, e)
		if err != nil {
			return nil, err
		}
		json.Unmarshal(body, &resp.NotebookResp)
	} else if modelArtsType == l.svcCtx.Config.NanjingModelArtsType {
		AK := l.svcCtx.Config.AK
		SK := l.svcCtx.Config.SK
		NanjingModelArtsUrl := l.svcCtx.Config.NanjingModelArtsUrl
		XProjectId := l.svcCtx.Config.XProjectId
		s := core.Signer{
			Key:    AK,
			Secret: SK,
		}
		reqByte, err := json.Marshal(in)
		r, err := http.NewRequest("POST", NanjingModelArtsUrl+"v1/"+in.ProjectId+"/notebooks"+in.Id+"/stop",
			bytes.NewBuffer(reqByte))
		if err != nil {
			fmt.Println(err)
		}
		r.Header.Add("content-type", "application/json;charset=UTF-8")
		r.Header.Add("X-Project-Id", XProjectId)
		r.Header.Add("x-stage", "RELEASE")
		s.Sign(r)
		client := http.DefaultClient
		res, err := client.Do(r)
		if err != nil {
			fmt.Println(err)
		}
		defer res.Body.Close()
		body, err := ioutil.ReadAll(res.Body)
		if err != nil {
			fmt.Println(err)
		}
		json.Unmarshal(body, &resp)
	}
	return &resp, nil
}
