package main

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/config"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/logic"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/server"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	commonConfig "PCM/common/config"
	"PCM/common/interceptor/rpcserver"
	"flag"
	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/zrpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "adaptor/PCM-AI/PCM-MODELARTS/rpc/etc/pcmmodelarts.yaml", "the config file")

func main() {

	flag.Parse()

	var bootstrapConfig commonConfig.BootstrapConfig
	conf.MustLoad(*configFile, &bootstrapConfig)

	//解析业务配置
	var c config.Config
	nacosConfig := bootstrapConfig.NacosConfig

	serviceConfigContent := nacosConfig.InitConfig(func(data string) {
		err := conf.LoadFromYamlBytes([]byte(data), &c)
		if err != nil {
			panic(err)
		}
	})
	err := conf.LoadFromYamlBytes([]byte(serviceConfigContent), &c)
	if err != nil {
		panic(err)
	}
	// start log component
	logx.MustSetup(c.LogConf)
	// 注册到nacos
	nacosConfig.Discovery(&c.RpcServerConf)

	ctx := svc.NewServiceContext(c)
	ctx.Cron.Start()

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		modelarts.RegisterModelArtsServer(grpcServer, server.NewModelArtsServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})

	//rpc log
	s.AddUnaryInterceptors(rpcserver.LoggerInterceptor)

	defer s.Stop()

	logx.Infof("Starting rpc server at %s...\n", c.ListenOn)
	logic.InitCron(ctx)
	s.Start()
}
