package main

import (
	"flag"
	"fmt"

	"PCM/adaptor/PCM-AI/PCM-MS/rpc/internal/config"
	"PCM/adaptor/PCM-AI/PCM-MS/rpc/internal/server"
	"PCM/adaptor/PCM-AI/PCM-MS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-MS/rpc/ms"

	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/zrpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "etc/pcmms.yaml", "the config file")

func main() {
	flag.Parse()

	var c config.Config
	conf.MustLoad(*configFile, &c)
	ctx := svc.NewServiceContext(c)

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		ms.RegisterAgentServiceServer(grpcServer, server.NewAgentServiceServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})
	defer s.Stop()

	fmt.Printf("Starting rpc server at %s...\n", c.ListenOn)
	s.Start()
}
