package config

import (
	commonConfig "PCM/common/config"
	"flag"
	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"github.com/zeromicro/go-zero/zrpc"
)

type Config struct {
	zrpc.RpcServerConf
	OctopusConfig  OctopusConfig
	OctopusApi     OctopusApi
	PcmCoreRpcConf zrpc.RpcClientConf
	LogConf        logx.LogConf
	RedisConf      redis.RedisConf
}

var configFile = flag.String("f", "adaptor/PCM-AI/PCM-OCTOPUS/rpc/etc/octopus.yaml", "the config file")

var Cfg = getConfig()

func getConfig() Config {
	var bootstrapConfig commonConfig.BootstrapConfig
	conf.MustLoad(*configFile, &bootstrapConfig)

	//解析业务配置
	var c Config
	nacosConfig := bootstrapConfig.NacosConfig

	serviceConfigContent := nacosConfig.InitConfig(func(data string) {
		err := conf.LoadFromYamlBytes([]byte(data), &c)
		if err != nil {
			panic(err)
		}
	})
	err := conf.LoadFromYamlBytes([]byte(serviceConfigContent), &c)
	if err != nil {
		panic(err)
	}
	// start log component
	logx.MustSetup(c.LogConf)
	// 注册到nacos
	nacosConfig.Discovery(&c.RpcServerConf)

	return c
}
