package logic

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/common"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"
	"PCM/common/tool"
	"context"

	"github.com/zeromicro/go-zero/core/logx"
)

type DownloadModelVersionLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDownloadModelVersionLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DownloadModelVersionLogic {
	return &DownloadModelVersionLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *DownloadModelVersionLogic) DownloadModelVersion(in *octopus.DownloadModelVersionReq) (*octopus.DownloadModelVersionResp, error) {
	resp := &octopus.DownloadModelVersionResp{}

	var url_prefix = common.OctopusUrls[in.Platform]
	var reqUrl = url_prefix + l.svcCtx.Config.OctopusApi.DownloadModelVersion

	token := common.GetToken(in.Platform)

	req := tool.GetACHttpRequest()
	_, err := req.
		SetHeader("Authorization", "Bearer "+token).
		SetPathParam("modelId", in.ModelId).
		SetPathParam("version", in.Version).
		SetQueryString("domain=" + in.Domain).
		SetResult(resp).
		Get(reqUrl)

	if err != nil {
		return nil, err
	}
	return resp, nil
}
