package logic

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/common"
	"PCM/common/tool"
	"context"
	"strconv"

	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetAlgorithmListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetAlgorithmListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetAlgorithmListLogic {
	return &GetAlgorithmListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetAlgorithmListLogic) GetAlgorithmList(in *octopus.GetAlgorithmListReq) (*octopus.GetAlgorithmListResp, error) {
	resp := &octopus.GetAlgorithmListResp{}

	var url_prefix = common.OctopusUrls[in.Platform]
	var reqUrl = url_prefix + l.svcCtx.Config.OctopusApi.GetAlgorithmList

	token := common.GetToken(in.Platform)

	req := tool.GetACHttpRequest()
	_, err := req.
		SetHeader("Authorization", "Bearer "+token).
		SetQueryString("pageIndex="+strconv.Itoa(int(in.PageIndex))).
		SetQueryString("pageSize="+strconv.Itoa(int(in.PageSize))).
		SetPathParam("algorithmId", in.AlgorithmId).
		SetResult(resp).
		Get(reqUrl)

	if err != nil {
		return nil, err
	}

	return resp, nil
}
