package logic

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/common"
	"PCM/common/tool"
	"context"

	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetAlgorithmLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetAlgorithmLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetAlgorithmLogic {
	return &GetAlgorithmLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetAlgorithmLogic) GetAlgorithm(in *octopus.GetAlgorithmReq) (*octopus.GetAlgorithmResp, error) {
	resp := &octopus.GetAlgorithmResp{}

	var url_prefix = common.OctopusUrls[in.Platform]
	var reqUrl = url_prefix + l.svcCtx.Config.OctopusApi.GetAlgorithm

	token := common.GetToken(in.Platform)

	req := tool.GetACHttpRequest()
	_, err := req.
		SetHeader("Authorization", "Bearer "+token).
		SetPathParam("algorithmId", in.AlgorithmId).
		SetPathParam("version", in.Version).
		SetResult(resp).
		Get(reqUrl)

	if err != nil {
		return nil, err
	}
	return resp, nil
}
