package logic

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/common"
	"PCM/common/tool"
	"context"
	"strconv"

	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetModelDeployEventLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetModelDeployEventLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetModelDeployEventLogic {
	return &GetModelDeployEventLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetModelDeployEventLogic) GetModelDeployEvent(in *octopus.GetModelDeployEventReq) (*octopus.GetModelDeployEventResp, error) {
	resp := &octopus.GetModelDeployEventResp{}

	var url_prefix = common.OctopusUrls[in.Platform]
	var reqUrl = url_prefix + l.svcCtx.Config.OctopusApi.GetModelDeployEvent

	token := common.GetToken(in.Platform)

	req := tool.GetACHttpRequest()
	_, err := req.
		SetHeader("Authorization", "Bearer "+token).
		SetQueryString("id=" + in.Id).
		SetQueryString("pageIndex=" + strconv.Itoa(int(in.PageIndex))).
		SetQueryString("pageSize=" + strconv.Itoa(int(in.PageSize))).
		SetQueryString("isMain=" + strconv.FormatBool(in.IsMain)).
		SetResult(resp).
		Get(reqUrl)

	if err != nil {
		return nil, err
	}

	return resp, nil
}
