package logic

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/common"
	"PCM/common/tool"
	"context"
	"strconv"

	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/internal/svc"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetModelVersionListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetModelVersionListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetModelVersionListLogic {
	return &GetModelVersionListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetModelVersionListLogic) GetModelVersionList(in *octopus.GetModelVersionListReq) (*octopus.GetModelVersionListResp, error) {
	resp := &octopus.GetModelVersionListResp{}

	var url_prefix = common.OctopusUrls[in.Platform]
	var reqUrl = url_prefix + l.svcCtx.Config.OctopusApi.GetModelVersionList

	token := common.GetToken(in.Platform)

	req := tool.GetACHttpRequest()
	_, err := req.
		SetHeader("Authorization", "Bearer "+token).
		SetPathParam("modelId", in.ModelId).
		SetQueryString("pageIndex=" + strconv.Itoa(int(in.PageIndex))).
		SetQueryString("pageSize=" + strconv.Itoa(int(in.PageSize))).
		SetResult(resp).
		Get(reqUrl)

	if err != nil {
		return nil, err
	}

	return resp, nil
}
