package image

import (
	result2 "PCM/common/result"
	"bufio"
	"context"
	"encoding/base64"
	"fmt"
	types2 "github.com/docker/docker/api/types"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/rest/httpx"
	"io/ioutil"
	"k8s.io/apimachinery/pkg/util/json"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"sync"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
)

var dir, _ = os.Getwd()
var uploadPath = filepath.Join(dir, "uploads")

func ChunkImageHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		size, _ := strconv.ParseInt(r.PostFormValue("size"), 10, 64)
		hash := r.PostFormValue("hash")
		name := r.PostFormValue("name")
		// 对比合并请求的文件大小和已上传文件夹大小
		toSize, _ := getDirSize(filepath.Join(uploadTempPath, hash))
		if size != toSize {
			fmt.Fprintf(w, "文件上传错误")
		}
		chunksPath := filepath.Join(uploadTempPath, hash)
		files, _ := ioutil.ReadDir(chunksPath)
		// 将文件根据索引序号排序
		filesSort := make(map[string]string)
		for _, f := range files {
			nameArr := strings.Split(f.Name(), "-")
			filesSort[nameArr[1]] = f.Name()
		}
		saveFile := filepath.Join(uploadPath, name)
		if exists, _ := PathExists(saveFile); exists {
			os.Remove(saveFile)
		}
		fs, _ := os.OpenFile(saveFile, os.O_CREATE|os.O_RDWR|os.O_APPEND, os.ModeAppend|os.ModePerm)
		var wg sync.WaitGroup
		filesCount := len(files)
		if filesCount != len(filesSort) {
			fmt.Fprintf(w, "文件上传错误2")
		}
		wg.Add(filesCount)
		for i := 0; i < filesCount; i++ {
			// 这里一定要注意按顺序读取不然文件就会损坏
			fileName := filepath.Join(chunksPath, filesSort[strconv.Itoa(i)])
			data, err := ioutil.ReadFile(fileName)
			fmt.Println(err)
			fs.Write(data)

			wg.Done()
		}
		wg.Wait()
		os.RemoveAll(chunksPath)
		defer fs.Close()

		// 加载镜像文件到docker
		fileInfo, err := os.Open(filepath.Join(uploadPath, name))
		reader := bufio.NewReader(fileInfo)
		if err != nil {
			return
		}
		body, err := svcCtx.DockerClient.ImageLoad(context.Background(), reader, false)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}
		bytes, err := ioutil.ReadAll(body.Body)
		loadBody := LoadBody{}
		err = json.Unmarshal(bytes, &loadBody)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}
		imageName := strings.TrimSpace(loadBody.Stream[13:])
		privateImageName := "hub.jcce.dev:18445/repository/pcm/" + imageName
		// 给镜像打上私有仓库的tag
		err = svcCtx.DockerClient.ImageTag(context.Background(), imageName, privateImageName)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}
		// 推送镜像到registry
		authConfig := types2.AuthConfig{
			Username: "admin",
			Password: "Nudt@123",
		}
		authConfigBytes, err := json.Marshal(authConfig)
		authStr := base64.URLEncoding.EncodeToString(authConfigBytes)
		_, err = svcCtx.DockerClient.ImagePush(context.Background(), privateImageName, types2.ImagePushOptions{RegistryAuth: authStr})
		if err != nil {
			logx.Error(err.Error())
			return
		}
		println("传输完成！")
		// 删除本地镜像 避免存储资源浪费
		_, err = svcCtx.DockerClient.ImageRemove(context.Background(), privateImageName, types2.ImageRemoveOptions{})
		if err != nil {
			return
		}
		result2.HttpResult(r, w, nil, err)
	}
}

// DirSize 获取整体文件夹大小
func getDirSize(path string) (int64, error) {
	var size int64
	err := filepath.Walk(path, func(_ string, info os.FileInfo, err error) error {
		if !info.IsDir() {
			size += info.Size()
		}
		return err
	})
	return size, err
}
