package image

import (
	result2 "PCM/common/result"
	"bufio"
	"fmt"
	"io"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"syscall"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
)

type LoadBody struct {
	Stream string `json:"stream"`
}

var uploadTempPath = filepath.Join(uploadPath, "temp")

func UploadImageHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		file, fileHeader, err := r.FormFile("file")
		index := r.PostFormValue("index")
		hash := r.PostFormValue("hash")
		// 合并路径
		chunksPath := filepath.Join(uploadTempPath, hash)
		// 文件路径
		filePath := filepath.Join(chunksPath, hash+"-"+index)
		// 检查临时文件夹是否存在
		isPathExists, err := PathExists(chunksPath)
		if !isPathExists {
			err = os.MkdirAll(chunksPath, os.ModePerm)
		}
		// 检查文件是否存在
		exists, err := PathExists(filePath)
		// 文件存在 进行断点续传
		if exists {
			fileInfo, _ := os.Stat(filePath)
			if fileInfo.Size() == fileHeader.Size {
				result2.HttpResult(r, w, nil, err)
				return
			}
			start := strconv.Itoa(int(fileInfo.Size()))
			oldFile, _ := os.OpenFile(filePath, os.O_CREATE|os.O_WRONLY, os.ModePerm)
			defer file.Close()
			count, _ := strconv.ParseInt(start, 10, 64)
			fmt.Println("已上传:", count)
			// 设置读，写的偏移量
			file.Seek(count, 0)
			oldFile.Seek(count, 0)
			data := make([]byte, 1024, 1024)
			for {
				total, err := file.Read(data)
				if err == io.EOF {
					fmt.Println("文件复制完毕")
					break
				}
				oldFile.Write(data[:total])

			}
			// 文件不存在 直接上传
		} else {
			destFile, _ := os.OpenFile(filepath.Join(chunksPath, hash+"-"+index), syscall.O_CREAT|syscall.O_WRONLY, 0777)
			reader := bufio.NewReader(file)
			writer := bufio.NewWriter(destFile)
			buf := make([]byte, 1024*1024) // 1M buf
			for {
				n, err := reader.Read(buf)
				if err == io.EOF {
					writer.Flush()
					break
				} else if err != nil {
					return
				} else {
					writer.Write(buf[:n])
				}
			}
			defer file.Close()
			defer destFile.Close()
		}

		result2.HttpResult(r, w, nil, err)
	}
}

// PathExists 判断文件夹是否存在
func PathExists(path string) (bool, error) {
	_, err := os.Stat(path)
	if err == nil {
		return true, nil
	}
	if os.IsNotExist(err) {
		return false, nil
	}
	return false, err
}
