package ai

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/result"
	"PCM/common/tool"
	"PCM/common/xerr"
	"context"
	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"k8s.io/apimachinery/pkg/util/json"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type CreateTaskLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewCreateTaskLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreateTaskLogic {
	return &CreateTaskLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *CreateTaskLogic) CreateTask(req *types.ImportTaskDataReq) (resp *types.ImportTaskDataResp, err error) {
	modelartsReq := &modelarts.ImportTaskDataReq{}
	err = copier.CopyWithOption(modelartsReq, req, copier.Option{Converters: tool.Converters})
	ImportTaskDataResp, err := l.svcCtx.ModelArtsRpc.CreateTask(l.ctx, modelartsReq)
	if err != nil {
		return nil, errors.Wrapf(xerr.NewErrMsg("Failed to get db DataSet list"), "Failed to get db DataSet list err : %v ,req:%+v", err, req)
	}
	marshal, err := json.Marshal(&ImportTaskDataResp)
	if err != nil {
		return nil, result.NewDefaultError(err.Error())
	}
	json.Unmarshal(marshal, &resp)
	err = copier.CopyWithOption(&resp, &ImportTaskDataResp, copier.Option{IgnoreEmpty: true, DeepCopy: true, Converters: tool.Converters})
	return resp, nil
}
