package ai

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelarts"
	"PCM/common/result"
	"PCM/common/tool"
	"context"
	"github.com/jinzhu/copier"
	"k8s.io/apimachinery/pkg/util/json"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListServicesLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewListServicesLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListServicesLogic {
	return &ListServicesLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ListServicesLogic) ListServices(req *types.ListServicesReq) (resp *types.ListServicesResp, err error) {
	modelartsReq := &modelarts.ListServicesReq{}
	err = copier.CopyWithOption(modelartsReq, req, copier.Option{Converters: tool.Converters})
	ListServicesResp, err := l.svcCtx.ModelArtsRpc.ListServices(l.ctx, modelartsReq)
	if err != nil {
		return nil, result.NewDefaultError(err.Error())
	}
	marshal, err := json.Marshal(&ListServicesResp)
	if err != nil {
		return nil, result.NewDefaultError(err.Error())
	}
	json.Unmarshal(marshal, &resp)
	err = copier.CopyWithOption(&resp, &ListServicesResp, copier.Option{IgnoreEmpty: true, DeepCopy: true, Converters: tool.Converters})
	return resp, nil
}
