package core

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"
	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcAC"
	"context"
	"log"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetComputingPowerLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetComputingPowerLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetComputingPowerLogic {
	return &GetComputingPowerLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetComputingPowerLogic) GetComputingPower() (resp *types.CpResp, err error) {

	apiResp := types.CpResp{}

	//启智章鱼资源算力
	octopusCpReq := &octopus.ResourceReq{}
	octopusCpResp, err := l.svcCtx.OctopusRpc.GetComputingPower(l.ctx, octopusCpReq)
	if err != nil {
		log.Println("OctopusRpc 算力请求失败", err)
	}

	//曙光账号算力
	acCpReq := &hpcAC.ResourceReq{}
	acCpResp, err := l.svcCtx.ACRpc.GetComputingPower(l.ctx, acCpReq)
	if err != nil {
		log.Println("ACRpc 算力请求失败", err)
	}

	computingPowerAggregated := acCpResp.POpsAtFp16 + octopusCpResp.POpsAtFp16
	apiResp.POpsAtFp16 = computingPowerAggregated
	return &apiResp, nil
}
