package core

import (
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopus"
	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcAC"
	"context"
	"log"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetGeneralInfoLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetGeneralInfoLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetGeneralInfoLogic {
	return &GetGeneralInfoLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *GetGeneralInfoLogic) GetGeneralInfo() (resp *types.GiResp, err error) {
	apiResp := types.GiResp{}

	//启智章鱼资源统计
	octopusGiReq := &octopus.ResourceReq{}
	octopusGiResp, err := l.svcCtx.OctopusRpc.GetGeneralInfo(l.ctx, octopusGiReq)
	if err != nil {
		log.Println("OctopusRpc 资源请求失败", err)
	}

	//曙光账号资源统计
	acGiReq := &hpcAC.ResourceReq{}
	acGiResp, err := l.svcCtx.ACRpc.GetGeneralInfo(l.ctx, acGiReq)
	if err != nil {
		log.Println("ACRpc 资源请求失败", err)
	}

	cpu := octopusGiResp.CpuCoreNum + acGiResp.CpuCoreNum
	storage := acGiResp.StorageInGib
	mem := octopusGiResp.MemoryInGib + acGiResp.MemoryInGib

	apiResp.StorageInGib = storage
	apiResp.CpuNum = cpu
	apiResp.MemoryInGib = mem

	return &apiResp, nil
}
