package core

import (
	"PCM/adaptor/PCM-CORE/model"
	"context"
	"k8s.io/apimachinery/pkg/util/json"
	"time"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type ScheduleTaskByYamlLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewScheduleTaskByYamlLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ScheduleTaskByYamlLogic {
	return &ScheduleTaskByYamlLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ScheduleTaskByYamlLogic) ScheduleTaskByYaml(req *types.ScheduleTaskByYamlReq) error {

	bytes, err := json.Marshal(req)
	if err != nil {
		return err
	}
	// construct task info
	taskModel := model.Task{
		Status:      "Saved",
		Description: req.Description,
		Name:        req.Name,
		YamlString:  string(bytes),
		StartTime:   time.Now(),
		CreatedTime: time.Now(),
		UpdatedTime: time.Now(),
	}
	// save the task in mysql and return id
	tx := l.svcCtx.DbEngin.Create(&taskModel)
	if tx.Error != nil {
		return tx.Error
	}

	// push message into topic
	for _, task := range req.Tasks {
		task.TaskId = taskModel.Id
		reqMessage, err := json.Marshal(task)
		if err != nil {
			logx.Error(err)
			return err
		}
		switch task.ServiceName {
		case "kubeNative":
			l.svcCtx.ScheduleCloudClient.Push(string(reqMessage))
		case "ac", "th":
			l.svcCtx.ScheduleHpcClient.Push(string(reqMessage))
		case "modelArts":
			l.svcCtx.ScheduleAiClient.Push(string(reqMessage))
		}
	}
	return nil
}
