package core

import (
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcAC"
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/hpcTH"
	"PCM/common/tool"
	"PCM/common/xerr"
	"context"
	"github.com/jinzhu/copier"
	"github.com/pkg/errors"
	"strconv"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type SubmitJobLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewSubmitJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SubmitJobLogic {
	return &SubmitJobLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *SubmitJobLogic) SubmitJob(req *types.SubmitJobReq) (resp *types.SubmitJobResp, err error) {

	coreResp := &types.SubmitJobResp{}

	version := req.SlurmVersion
	switch version {
	case "ac":
		acReq := &hpcAC.SubmitJobReq{}
		err = copier.CopyWithOption(acReq, req, copier.Option{Converters: tool.Converters})
		acResp, err := l.svcCtx.ACRpc.SubmitJob(l.ctx, acReq)

		if err != nil {
			return nil, errors.Wrapf(xerr.NewErrMsg("Failed to submit job to AC"), "Failed to submit job to Shuguang err : %v ,req:%+v", err, req)
		}

		if acResp.Data == "" {
			return nil, errors.Wrapf(xerr.NewErrMsg("Failed to submit job to AC, job id is empty"), "Failed to submit job to Shuguang err : %v", err)
		}

		coreResp.Msg = acResp.Msg
		coreResp.Code = acResp.Code
		coreResp.Data = acResp.Data

	case "th":
		thReq := &hpcTH.SubmitJobReq{}
		err = copier.CopyWithOption(thReq, req, copier.Option{Converters: tool.Converters})
		tianheResp, err := l.svcCtx.THRpc.SubmitJob(l.ctx, thReq)

		if err != nil {
			return nil, errors.Wrapf(xerr.NewErrMsg("Failed to submit job to Tianhe"), "Failed to submit job to Tianhe err : %v ,req:%+v", err, req)
		}

		coreResp.Msg = "success"
		coreResp.Code = strconv.Itoa(int(tianheResp.SubmitResponseMsg[0].ErrorCode))
		coreResp.Data = strconv.Itoa(int(tianheResp.SubmitResponseMsg[0].JobId))
	}

	return coreResp, nil
}
