package core

import (
	"PCM/adaptor/PCM-CORE/model"
	"PCM/common/enum"
	"context"
	"strings"

	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type TaskListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewTaskListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *TaskListLogic {
	return &TaskListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *TaskListLogic) TaskList() (resp *types.TaskListResp, err error) {
	resp = &types.TaskListResp{}

	// 查询任务数据
	var tasks []model.Task
	tx := l.svcCtx.DbEngin.Find(&tasks)
	if tx.Error != nil {
		logx.Error(err)
		return nil, tx.Error
	}
	if len(tasks) == 0 {
		return nil, nil
	}
	for _, task := range tasks {
		tx := l.svcCtx.DbEngin.Raw("SELECT CONCAT_WS(',',GROUP_CONCAT(DISTINCT h.service_name) ,GROUP_CONCAT(DISTINCT  a.service_name) ,GROUP_CONCAT(DISTINCT c.service_name))as service_name from task t left join hpc h on t.id  = h.task_id left join cloud c  on t.id  = c.task_id left join ai a on t.id  = a.task_id where t.id  = ?", task.Id).Scan(&task.ServiceName)
		if tx.Error != nil {
			logx.Error(err)
			return nil, tx.Error
		}
		// 承接方转义
		if task.ServiceName != "" {
			var names []string
			servicesName := strings.Split(task.ServiceName, ",")
			for _, name := range servicesName {
				names = append(names, enum.Partner(name).String())
			}
			task.ServiceName = strings.Join(names, ",")
		}
		resp.Tasks = append(resp.Tasks, types.Task{
			ServiceName:   task.ServiceName,
			Name:          task.Name,
			Strategy:      int(task.Strategy),
			SynergyStatus: enum.SynergyStatus(task.SynergyStatus).String(),
			Status:        task.Status,
		})

	}
	// 查询总运行时长
	tx = l.svcCtx.DbEngin.Raw("select sum(running_time)/3600 as total_run_time from (select sum(running_time) as running_time from hpc union all select sum(running_time) as running_time from cloud union all select sum(running_time) as running_time from ai) runtime").Scan(&resp.TotalRunTime)
	if tx.Error != nil {
		logx.Error(err)
		return nil, tx.Error
	}
	// 运行卡时数
	tx = l.svcCtx.DbEngin.Model(&model.Hpc{}).Select("(CASE WHEN SUM(running_time * card_count)/3600 IS NULL THEN 0 ELSE SUM(running_time * card_count)/3600 END )as cardTime").Find(&resp.CardTime)
	if tx.Error != nil {
		return nil, tx.Error
	}

	// 运行任务合计数
	resp.TotalCount = len(tasks)
	return resp, nil
}
