package hpc

import (
	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"
	"PCM/common/enum"
	"context"
	"strings"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListHistoryJobLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewListHistoryJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListHistoryJobLogic {
	return &ListHistoryJobLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ListHistoryJobLogic) ListHistoryJob(req *types.ListHistoryJobReq) (resp *types.ListHistoryJobResp, err error) {

	resp = &types.ListHistoryJobResp{}

	var tasks []types.HistoryJob
	// 查询任务数据

	tx := l.svcCtx.DbEngin.Raw("SELECT h.service_name as SlurmVersion,h.name,h.start_time as JobStartTime,h.running_time as JobRunTime,t.status as StateofJob from hpc h join task t on t.id  = h.task_id and t.status = 'Completed'").Scan(&tasks)
	if tx.Error != nil {
		logx.Error(err)
		return nil, tx.Error
	}
	for _, task := range tasks {
		// 承接方转义
		if task.SlurmVersion != "" {
			var names []string
			servicesName := strings.Split(task.SlurmVersion, ",")
			for _, name := range servicesName {
				names = append(names, enum.Partner(name).String())
			}
			task.SlurmVersion = strings.Join(names, ",")
		}
		resp.HistoryJobs = append(resp.HistoryJobs, types.HistoryJob{
			SlurmVersion: task.SlurmVersion,
			Name:         task.Name,
			JobStartTime: task.JobStartTime,
			JobRunTime:   task.JobRunTime,
			StateofJob:   task.StateofJob,
		})

	}
	resp.Code = 200
	resp.Msg = "success"
	resp.RecordCount = int32(len(resp.HistoryJobs))

	return resp, nil
}
