package image

import (
	"PCM/adaptor/PCM-CORE/api/internal/svc"
	"PCM/adaptor/PCM-CORE/api/internal/types"
	"PCM/common/tool"
	"context"
	"encoding/json"
	"fmt"
	"github.com/zeromicro/go-zero/core/logx"
)

type ImageListLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

type ImageTags struct {
	Name string   `json:"name"`
	Tags []string `json:"tags" copier:"tags"`
}

func NewImageListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ImageListLogic {
	return &ImageListLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

func (l *ImageListLogic) ImageList() (resp *types.ImageListResp, err error) {

	// 获取镜像列表
	url := fmt.Sprintf("%s/repository/%s/v2/_catalog", l.svcCtx.Config.NexusUrl, "pcm")
	bytes, err := tool.HttpClient("GET", url, nil, "")
	json.Unmarshal(bytes, &resp)
	result := types.ImageListResp{}
	for _, image := range resp.Repositories {
		// 获取镜像的tag列表
		url := fmt.Sprintf("%s/repository/%s/v2/%s/tags/list", l.svcCtx.Config.NexusUrl, "pcm", image)
		bytes, err := tool.HttpClient("GET", url, nil, "")
		if err != nil {
			return nil, err
		}
		tags := ImageTags{}
		json.Unmarshal(bytes, &tags)
		// 拼接镜像名称
		for _, tag := range tags.Tags {
			var name string
			if tag != "latest" {
				name = "hub.jcce.dev:18445/" + image + ":" + tag
			} else {
				name = "hub.jcce.dev:18445/" + image
			}
			result.Repositories = append(result.Repositories, name)
		}
	}
	return &result, nil
}
