package svc

import (
	"PCM/adaptor/PCM-AI/PCM-MODELARTS/rpc/modelartsclient"
	"PCM/adaptor/PCM-AI/PCM-OCTOPUS/rpc/octopusclient"
	"PCM/adaptor/PCM-CORE/api/internal/config"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcacclient"
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/hpcthclient"
	"PCM/adaptor/PCM-STORAGE/PCM-CEPH/rpc/cephclient"
	"github.com/docker/docker/client"
	"github.com/go-redis/redis/v8"
	"github.com/robfig/cron/v3"
	"github.com/zeromicro/go-queue/kq"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/zrpc"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/schema"
)

type ServiceContext struct {
	Config              config.Config
	RedisClient         *redis.Client
	ScheduleHpcClient   *kq.Pusher
	ScheduleCloudClient *kq.Pusher
	ScheduleAiClient    *kq.Pusher
	Cron                *cron.Cron
	ModelArtsRpc        modelartsclient.ModelArts
	DbEngin             *gorm.DB
	ACRpc               hpcacclient.HpcAC
	THRpc               hpcthclient.HpcTH
	OctopusRpc          octopusclient.Octopus
	CephRpc             cephclient.Ceph
	DockerClient        *client.Client
}

func NewServiceContext(c config.Config) *ServiceContext {
	//启动Gorm支持
	dbEngin, _ := gorm.Open(mysql.Open(c.DB.DataSource), &gorm.Config{
		NamingStrategy: schema.NamingStrategy{
			SingularTable: true, // 使用单数表名，启用该选项，此时，`User` 的表名应该是 `t_user`
		},
	})
	dockerClient, err := client.NewClientWithOpts()
	if err != nil {
		logx.Error(err.Error())
		return nil
	}
	return &ServiceContext{
		Cron:    cron.New(cron.WithSeconds()),
		DbEngin: dbEngin,
		Config:  c,
		RedisClient: redis.NewClient(&redis.Options{
			Addr:     c.Redis.Host,
			Password: c.Redis.Pass,
		}),
		ScheduleHpcClient:   kq.NewPusher(c.KqProducerConf.Brokers, c.KqProducerConf.HpcTopic),
		ScheduleCloudClient: kq.NewPusher(c.KqProducerConf.Brokers, c.KqProducerConf.CloudTopic),
		ScheduleAiClient:    kq.NewPusher(c.KqProducerConf.Brokers, c.KqProducerConf.AiTopic),
		ModelArtsRpc:        modelartsclient.NewModelArts(zrpc.MustNewClient(c.ModelArtsRpcConf)),
		CephRpc:             cephclient.NewCeph(zrpc.MustNewClient(c.CephRpcConf)),
		ACRpc:               hpcacclient.NewHpcAC(zrpc.MustNewClient(c.ACRpcConf)),
		//THRpc:               hpcthclient.NewHpcTH(zrpc.MustNewClient(c.THRpcConf)),
		OctopusRpc:   octopusclient.NewOctopus(zrpc.MustNewClient(c.OctopusRpcConf)),
		DockerClient: dockerClient,
	}
}
