package logic

import (
	"PCM/adaptor/PCM-CORE/model"
	"PCM/adaptor/PCM-CORE/rpc/internal/svc"
	"PCM/adaptor/PCM-CORE/rpc/pcmCore"
	"PCM/common/tool"
	"context"

	"github.com/zeromicro/go-zero/core/logx"
)

type InfoListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewInfoListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *InfoListLogic {
	return &InfoListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// InfoList
func (l *InfoListLogic) InfoList(in *pcmCore.InfoListReq) (*pcmCore.InfoListResp, error) {
	result := pcmCore.InfoListResp{}
	// 查询云智超中的数据列表
	switch in.Kind {
	case "hpc":
		var hpcModelList []model.Hpc
		tx := l.svcCtx.DbEngin.Where("service_name = ? AND status NOT IN ?", in.ServiceName, []string{"Succeed", "Completed"}).Find(&hpcModelList)
		if tx.Error != nil {
			return nil, tx.Error
		}
		var hpcInfoList []*pcmCore.HpcInfo
		tool.Convert(hpcModelList, &hpcInfoList)
		result.HpcInfoList = hpcInfoList
	case "cloud":
		var cloudModelList []model.Cloud
		tx := l.svcCtx.DbEngin.Where("service_name = ? AND status NOT IN ?", in.ServiceName, []string{"Succeed", "Completed"}).Find(&cloudModelList)
		if tx.Error != nil {
			return nil, tx.Error
		}
		var cloudInfoList []*pcmCore.CloudInfo
		tool.Convert(cloudModelList, &cloudInfoList)
		result.CloudInfoList = cloudInfoList
	case "ai":
		var aiModelList []model.Ai
		tx := l.svcCtx.DbEngin.Where("service_name = ? AND status NOT IN ?", in.ServiceName, []string{"Succeed", "Completed"}).Find(&aiModelList)
		if tx.Error != nil {
			return nil, tx.Error
		}
		var aiInfoList []*pcmCore.AiInfo
		tool.Convert(aiModelList, &aiInfoList)
		result.AiInfoList = aiInfoList
	}
	return &result, nil
}
