package logic

import (
	"PCM/adaptor/PCM-CORE/rpc/internal/svc"
	"PCM/adaptor/PCM-CORE/rpc/pcmCore"
	"context"
	"time"

	"github.com/zeromicro/go-zero/core/logx"
)

type ExternalInfo struct {
	TaskId        int64
	ExternalField string
	Name          string
	Status        string
	StartTime     time.Time
	RunningTime   int64
}
type SyncInfoLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewSyncInfoLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SyncInfoLogic {
	return &SyncInfoLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// SyncInfo Synchronous data information
func (l *SyncInfoLogic) SyncInfo(in *pcmCore.SyncInfoReq) (*pcmCore.SyncInfoResp, error) {
	db := l.svcCtx.DbEngin.Begin()
	if db.Error != nil {
		return nil, db.Error
	}
	switch in.Kind {
	case "cloud":
		for _, cloudInfo := range in.CloudInfoList {
			db.Exec("update cloud set status = ?,start_time = ?,running_time = ? where service_name = ? and task_id = ? and namespace = ? and name = ?",
				cloudInfo.Status, cloudInfo.StartTime, cloudInfo.RunningTime, in.ServiceName, cloudInfo.TaskId, cloudInfo.Namespace, cloudInfo.Name)
		}
	case "hpc":
		for _, hpcInfo := range in.HpcInfoList {
			tx := db.Exec("update hpc set status = ?,derived_es = ?,assoc_id = ?,exit_code = ?,version = ?,alloc_cpu = ?,alloc_nodes = ?,cluster = ?,block_id = ?,start_time = ?,running_time = ?,job_id = ? where service_name = ? and task_id = ? and name = ?",
				hpcInfo.Status, hpcInfo.DerivedEs, hpcInfo.AssocId, hpcInfo.ExitCode, hpcInfo.Version, hpcInfo.AllocCpu, hpcInfo.AllocNodes, hpcInfo.Cluster, hpcInfo.BlockId, hpcInfo.StartTime, hpcInfo.RunningTime, hpcInfo.JobId, in.ServiceName, hpcInfo.TaskId, hpcInfo.Name)
			print(tx.Error)
		}
	case "ai":
		for _, aiInfo := range in.AiInfoList {
			db.Exec("update ai set status = ?,start_time = ?,running_time = ?,project_id = ?,job_id = ? where service_name = ? and task_id = ? and name = ?",
				aiInfo.Status, aiInfo.StartTime, aiInfo.RunningTime, aiInfo.ProjectId, aiInfo.JobId, in.ServiceName, aiInfo.TaskId, aiInfo.Name)
		}
	}

	// 执行回滚或者提交操作
	defer func() {
		if p := recover(); p != nil {
			db.Rollback()
			logx.Error(p)
		} else if db.Error != nil {
			logx.Info("rollback")
			db.Rollback()
		} else {
			db = db.Commit()
			logx.Info("commit success")
		}
	}()
	return &pcmCore.SyncInfoResp{}, nil
}
