package logic

import (
	"PCM/adaptor/PCM-CORE/rpc/pcmcoreclient"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcAC"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/internal/svc"
	"PCM/common/enum"
	"PCM/common/tool"
	"context"
	"github.com/zeromicro/go-zero/core/logx"
)

func InitCron(svc *svc.ServiceContext) {
	submitJobLogic := NewSubmitJobLogic(context.Background(), svc)
	listLogic := NewListJobLogic(context.Background(), svc)
	svc.Cron.AddFunc("*/5  *  *  *  *  ?", func() {
		// 查询core端分发下来的任务列表
		infoReq := pcmcoreclient.InfoListReq{
			Kind:        "hpc",
			ServiceName: "ac",
		}
		infoList, err := svc.PcmCoreRpc.InfoList(context.Background(), &infoReq)
		if err != nil {
			logx.Error(err)
			return
		}
		// 提交任务
		submitJob(infoList, submitJobLogic)
		// 查询运行中的任务列表同步信息
		listJob, err := listLogic.ListJob(&hpcAC.ListJobReq{})
		if err != nil {
			logx.Error(err)
			return
		}
		for index, _ := range infoList.HpcInfoList {
			for _, job := range listJob.Jobs {
				if job.JobName == infoList.HpcInfoList[index].Name {
					infoList.HpcInfoList[index].JobId = job.JobId
					infoList.HpcInfoList[index].StartTime = job.JobStartTime
					infoList.HpcInfoList[index].RunningTime = int64(tool.RunTimeToSeconds(job.JobRunTime))
					infoList.HpcInfoList[index].Status = enum.AcStatus(job.JobStatus).String()
				}
			}
		}
		// 同步信息到core端
		if len(infoList.HpcInfoList) != 0 {
			syncInfoReq := pcmcoreclient.SyncInfoReq{
				Kind:        "hpc",
				ServiceName: "ac",
				HpcInfoList: infoList.HpcInfoList,
			}
			svc.PcmCoreRpc.SyncInfo(context.Background(), &syncInfoReq)
		}
	})
}

func submitJob(infoList *pcmcoreclient.InfoListResp, submitJobLogic *SubmitJobLogic) {
	for index, _ := range infoList.HpcInfoList {
		if infoList.HpcInfoList[index].Status == "Saved" {
			submitReq := hpcAC.SubmitJobReq{
				Appname:         "BASE",
				Apptype:         "BASIC",
				StrJobManagerID: 1638523853,
				MapAppJobInfo: &hpcAC.MapAppJobInfo{
					GAP_CMD_FILE:     infoList.HpcInfoList[index].CmdScript,
					GAP_NNODE:        "1",
					GAP_SUBMIT_TYPE:  "cmd",
					GAP_JOB_NAME:     infoList.HpcInfoList[index].Name,
					GAP_WORK_DIR:     infoList.HpcInfoList[index].WorkDir,
					GAP_QUEUE:        "debug2",
					GAP_NPROC:        "1",
					GAP_APPNAME:      "BASE",
					GAP_WALL_TIME:    infoList.HpcInfoList[index].WallTime,
					GAP_STD_OUT_FILE: "/public/home/zhijiang/test/testjob1/std.out.%j",
					GAP_STD_ERR_FILE: " /public/home/zhijiang/test/testjob1/std.err.%j",
				},
			}
			jobResult, _ := submitJobLogic.SubmitJob(&submitReq)
			if jobResult.Code == "0" {
				infoList.HpcInfoList[index].Status = "Pending"
				infoList.HpcInfoList[index].JobId = jobResult.Data
			} else {
				infoList.HpcInfoList[index].Status = "Failed"
				infoList.HpcInfoList[index].Result = jobResult.Msg
			}
		}
	}
}
