package logic

import (
	"context"
	"encoding/json"
	"io"
	"net/http"
	"net/url"
	"time"

	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/hpcAC"
	"PCM/adaptor/PCM-HPC/PCM-AC/rpc/internal/svc"
	"github.com/zeromicro/go-zero/core/logx"
)

type DeleteJobLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDeleteJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DeleteJobLogic {
	return &DeleteJobLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// DeleteJob delete job
func (l *DeleteJobLogic) DeleteJob(in *hpcAC.DeleteJobReq) (*hpcAC.DeleteJobResp, error) {
	// todo: add your logic here and delete this line
	var resp hpcAC.DeleteJobResp

	deleteJobUrl := "hpc/openapi/v2/jobs?"

	getTokenLogic := NewGetACTokenLogic(l.ctx, l.svcCtx)
	tokenResp, _ := getTokenLogic.GetACToken(&hpcAC.ACTokenReq{})
	token := tokenResp.GetData().Token

	c := http.Client{Timeout: time.Duration(3) * time.Second}

	params := url.Values{}
	params.Add("jobMethod", "5")
	params.Add("strJobInfoMap", in.StrJobInfoMap)

	reqUrl, err := http.NewRequest("DELETE", "https://api01.hpccube.com:65106/"+deleteJobUrl+params.Encode(), nil)

	if err != nil {
		return nil, err
	}

	reqUrl.Header.Add("token", token)

	respUrl, err := c.Do(reqUrl)
	if err != nil {
		return nil, err
	}

	body, err := io.ReadAll(respUrl.Body)
	if err != nil {
		return nil, err
	}
	if err = json.Unmarshal(body, &resp); err != nil {
		if resp.Code != "0" {
			resp.Data = nil
		} else {
			return nil, err
		}
	}
	return &resp, nil
}
