package logic

/*
#cgo LDFLAGS: -lslurmdb

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <slurm/slurm.h>
#include <slurm/slurmdb.h>
#include <slurm/slurm_errno.h>
#include <memory.h>
#include <malloc.h>

slurmdb_job_rec_t *get_all_slurmdb_job() {

    slurmdb_job_cond_t *job_cond = NULL;
	void *conn = slurmdb_connection_get();

	List joblist = slurmdb_jobs_get(conn, job_cond);

	uint16_t listsize = slurm_list_count(joblist);
	//qosinfo.record_count = size;
	slurmdb_job_rec_t *jobarray = malloc(listsize * sizeof(slurmdb_job_rec_t));

	ListIterator itr = slurm_list_iterator_create(joblist);

    slurmdb_job_rec_t *rec = NULL;
	int i = 0;
	while ((rec = slurm_list_next(itr))) {
  		jobarray[i] = *rec;
  		i++;
	}
	slurmdb_connection_close(&conn);
	slurm_list_destroy(joblist);
	//int arraysize = sizeof(jobarray);
	//printf("%d\n", arraysize);
	return jobarray;
}

slurmdb_job_rec_t *job_from_array(slurmdb_job_rec_t *job_rec_array, int i) {
  return (slurmdb_job_rec_t *) &(job_rec_array[i]);
}

slurmdb_job_rec_t *job_from_array_by_id(slurmdb_job_rec_t *job_rec_array, int job_id) {
	int i;
    int arraysize = sizeof(job_rec_array);
    for (i=0; i<arraysize; i++)
    {
         //printf("var: %d : %p : %d \n", var, &(job_rec_array[var]), (job_rec_array[var]).stats.cpu_min);
         //printf("%d \n",(job_rec_array[i]).stats.cpu_min);
         //printf("var: %d : %p : %d \n", var, &(job_rec_array[var]), (job_rec_array[var]).jobid);
         if (job_id==(job_rec_array[i]).jobid)
         {
			break;
         } else {
			return NULL;
         }
    }
	return (slurmdb_job_rec_t *) &(job_rec_array[i]);

}
int getLength(slurmdb_job_rec_t *job_rec_array) {
  return sizeof(job_rec_array);
}

*/
import "C"

import (
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/hpcTH"
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/internal/svc"
	"context"

	"github.com/zeromicro/go-zero/core/logx"
)

type ListHistoryJobLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewListHistoryJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ListHistoryJobLogic {
	return &ListHistoryJobLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// ListHistoryJob list all jobs from slurmdb
func (l *ListHistoryJobLogic) ListHistoryJob(in *hpcTH.ListHistoryJobReq) (*hpcTH.ListHistoryJobResp, error) {

	var dbJobResp hpcTH.ListHistoryJobResp
	jobInfos := C.get_all_slurmdb_job()
	size := uint32(C.getLength(jobInfos))
	dbJobResp.RecordCount = size
	for i := uint32(0); i < size; i++ {
		cJob := C.job_from_array(jobInfos, C.int(i))
		goJob := SlurmdbJobConvertCToGo(cJob)
		dbJobResp.HistoryJobs = append(dbJobResp.HistoryJobs, &goJob)
	}
	dbJobResp.Code = 200
	dbJobResp.Msg = "success"

	return &dbJobResp, nil
}

func SlurmdbJobConvertCToGo(cStructJob *C.slurmdb_job_rec_t) hpcTH.HistoryJob {
	var goStruct hpcTH.HistoryJob

	goStruct.Account = C.GoString(cStructJob.account)
	goStruct.Associd = uint32(cStructJob.associd)
	goStruct.AllocCpu = uint32(cStructJob.alloc_cpus)
	goStruct.AllocNodes = uint32(cStructJob.alloc_nodes)
	goStruct.Blockid = C.GoString(cStructJob.blockid)
	goStruct.Cluster = C.GoString(cStructJob.cluster)
	goStruct.DerivedEs = C.GoString(cStructJob.derived_es)
	goStruct.DerivedEc = uint32(cStructJob.derived_ec)
	goStruct.End = int64(cStructJob.end)
	goStruct.Elapsed = uint32(cStructJob.elapsed)
	goStruct.Eligible = int64(cStructJob.eligible)
	goStruct.Exitcode = uint32(cStructJob.exitcode)
	goStruct.Gid = uint32(cStructJob.gid)
	goStruct.Jobid = uint32(cStructJob.jobid)
	goStruct.Jobname = C.GoString(cStructJob.jobname)
	goStruct.Lft = uint32(cStructJob.lft)
	goStruct.Nodes = C.GoString(cStructJob.nodes)
	goStruct.Priority = uint32(cStructJob.priority)
	goStruct.Partition = C.GoString(cStructJob.partition)
	goStruct.Qosid = uint32(cStructJob.qosid)
	goStruct.Resvid = uint32(cStructJob.resvid)
	goStruct.Requid = uint32(cStructJob.requid)
	goStruct.ReqCpus = uint32(cStructJob.req_cpus)
	goStruct.ReqMem = uint32(cStructJob.req_mem)
	goStruct.Start = int64(cStructJob.start)
	goStruct.State = uint32(cStructJob.state)
	goStruct.SysCpuUsec = uint32(cStructJob.sys_cpu_usec)
	goStruct.SysCpuSec = uint32(cStructJob.sys_cpu_sec)
	goStruct.Submit = int64(cStructJob.submit)
	goStruct.Suspended = uint32(cStructJob.suspended)
	goStruct.ShowFull = uint32(cStructJob.show_full)

	//var stats_adrs []*pbslurm.SlurmdbStatsT
	//var stats pbslurm.SlurmdbStatsT
	//stat.ActCpufreq = float64(c_struct_job.stats.act_cpufreq)
	//stats.CpuMin = uint32((c_struct_job.stats).cpu_min)
	//stat.CpuAve = float64(c_struct_job.stats.cpu_ave)
	//stats_adrs = append(stats_adrs, &stats)
	//go_struct.Stats = stats

	return goStruct
}
