package logic

/*
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include<slurm/slurm.h>
#include<slurm/slurm_errno.h>

struct submit_response_msg *submit_job(struct job_descriptor *desc)
{

	struct submit_response_msg *resp_msg;
	if (slurm_submit_batch_job(desc,
				&resp_msg)) {
		return NULL;
	}
	return resp_msg;

}

void free_submit_response_msg(struct submit_response_msg *msg)
{
	 slurm_free_submit_response_response_msg(msg);
}

int update_job (struct job_descriptor *msg) {

	 return slurm_update_job (msg);
}
*/
import "C"

import (
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/hpcTH"
	"PCM/adaptor/PCM-HPC/PCM-TH/rpc/internal/svc"
	"context"
	"fmt"
	"unsafe"

	"github.com/zeromicro/go-zero/core/logx"
)

type SubmitJobLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewSubmitJobLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SubmitJobLogic {
	return &SubmitJobLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// Submit job
func (l *SubmitJobLogic) SubmitJob(in *hpcTH.SubmitJobReq) (*hpcTH.SubmitJobResp, error) {
	var cStruct C.struct_job_descriptor

	C.slurm_init_job_desc_msg(&cStruct)
	if in.Account != "" {
		accountS := C.CString(in.Account)
		defer C.free(unsafe.Pointer(accountS))
		cStruct.account = accountS
	}
	if in.AcctgFreq != "" {
		acctgFreqS := C.CString(in.AcctgFreq)
		defer C.free(unsafe.Pointer(acctgFreqS))
		cStruct.acctg_freq = acctgFreqS
	}
	if in.AllocNode != "" {
		allocNodeS := C.CString(in.AllocNode)
		defer C.free(unsafe.Pointer(allocNodeS))
		cStruct.alloc_node = allocNodeS
	}
	if in.AllocRespPort != 0 {
		cStruct.alloc_resp_port = C.uint16_t(in.AllocRespPort)
	}
	if in.AllocSid != 0 {
		cStruct.alloc_sid = C.uint32_t(in.AllocSid)
	}
	if len(in.Argv) > 0 {
		cStruct.argc = C.uint32_t(len(in.Argv))
		cArray := C.malloc(C.size_t(C.size_t(len(in.Argv)) * C.size_t(unsafe.Sizeof(uintptr(0)))))
		a := (*[1<<30 - 1]*C.char)(cArray)
		for i := 0; i < len(in.Argv); i++ {
			a[i] = C.CString(in.Argv[i].String())
		}
		cStruct.argv = (**C.char)(cArray)
		fmt.Printf("test\n")
	}

	if in.ArrayInx != "" {
		arrayInxS := C.CString(in.ArrayInx)
		defer C.free(unsafe.Pointer(arrayInxS))
		cStruct.array_inx = arrayInxS
	}
	if in.BeginTime != 0 {
		cStruct.begin_time = C.int64_t(in.BeginTime)
	}
	if in.CkptInterval != 0 {
		cStruct.ckpt_interval = C.uint16_t(in.CkptInterval)
	}
	if in.CkptDir != "" {
		ckptDirS := C.CString(in.CkptDir)
		defer C.free(unsafe.Pointer(ckptDirS))
		cStruct.ckpt_dir = ckptDirS
	}
	if in.Comment != "" {
		commentS := C.CString(in.Comment)
		defer C.free(unsafe.Pointer(commentS))
		cStruct.comment = commentS
	}
	if in.Contiguous != 0 {
		cStruct.contiguous = C.uint16_t(in.Contiguous)
	}
	if in.CpuBind != "" {
		cpuBindS := C.CString(in.CpuBind)
		defer C.free(unsafe.Pointer(cpuBindS))
		cStruct.cpu_bind = cpuBindS
	}
	if in.CpuBindType != 0 {
		cStruct.cpu_bind_type = C.uint16_t(in.CpuBindType)
	}
	if in.Dependency != "" {
		dependencyS := C.CString(in.Dependency)
		defer C.free(unsafe.Pointer(dependencyS))
		cStruct.dependency = dependencyS
	}
	if in.EndTime != 0 {
		cStruct.end_time = C.int64_t(in.EndTime)
	}
	if len(in.Environment) > 0 {
		cStruct.env_size = C.uint32_t(len(in.Environment))
		cArray := C.malloc(C.size_t(C.size_t(len(in.Environment)) * C.size_t(unsafe.Sizeof(uintptr(0)))))
		a := (*[1<<30 - 1]*C.char)(cArray)
		for i := 0; i < len(in.Environment); i++ {
			a[i] = C.CString(in.Environment[i].String())
			defer C.free(unsafe.Pointer(a[i]))
		}
		cStruct.environment = (**C.char)(cArray)
	} else {
		cStruct.env_size = 1
		cArray := C.malloc(C.size_t(C.size_t(1) * C.size_t(unsafe.Sizeof(uintptr(0)))))
		a := (*[1<<30 - 1]*C.char)(cArray)
		a[0] = C.CString("SLURM_GO_JOB=TRUE")
		defer C.free(unsafe.Pointer(a[0]))
		cStruct.environment = (**C.char)(cArray)

	}
	if in.ExcNodes != "" {
		excNodesS := C.CString(in.ExcNodes)
		defer C.free(unsafe.Pointer(excNodesS))
		cStruct.exc_nodes = excNodesS
	}
	if in.Features != "" {
		featuresS := C.CString(in.Features)
		defer C.free(unsafe.Pointer(featuresS))
		cStruct.features = featuresS
	}
	if in.GroupId != 0 {
		cStruct.group_id = C.uint32_t(in.GroupId)
	}
	if in.Immediate != 0 {
		cStruct.immediate = C.uint16_t(in.Immediate)
	}
	if in.JobId != 0 {
		cStruct.job_id = C.uint32_t(in.JobId)
	}
	if in.KillOnNodeFail != 0 {
		cStruct.kill_on_node_fail = C.uint16_t(in.KillOnNodeFail)
	}
	if in.Licenses != "" {
		licensesS := C.CString(in.Licenses)
		defer C.free(unsafe.Pointer(licensesS))
		cStruct.licenses = licensesS
	}
	if in.MailType != 0 {
		cStruct.mail_type = C.uint16_t(in.MailType)
	}
	if in.MailUser != "" {
		mailUserS := C.CString(in.MailUser)
		defer C.free(unsafe.Pointer(mailUserS))
		cStruct.mail_user = mailUserS
	}
	if in.MemBind != "" {
		memBindS := C.CString(in.MemBind)
		defer C.free(unsafe.Pointer(memBindS))
		cStruct.mem_bind = memBindS
	}
	if in.MemBindType != 0 {
		cStruct.mem_bind_type = C.uint16_t(in.MemBindType)
	}
	if in.Name != "" {
		nameS := C.CString(in.Name)
		defer C.free(unsafe.Pointer(nameS))
		cStruct.name = nameS
	}
	if in.Network != "" {
		networkS := C.CString(in.Network)
		defer C.free(unsafe.Pointer(networkS))
		cStruct.network = networkS
	}
	if in.Nice != 0 {
		cStruct.nice = C.uint16_t(in.Nice)
	}
	if in.NumTasks != 0 {
		cStruct.num_tasks = C.uint32_t(in.NumTasks)
	}
	if in.OpenMode != 0 {
		cStruct.open_mode = C.uint8_t(in.OpenMode)
	}
	if in.OtherPort != 0 {
		cStruct.other_port = C.uint16_t(in.OtherPort)
	}
	if in.Overcommit != 0 {
		cStruct.overcommit = C.uint8_t(in.Overcommit)
	}
	if in.Partition != "" {
		partitionS := C.CString(in.Partition)
		defer C.free(unsafe.Pointer(partitionS))
		cStruct.partition = partitionS
	}
	if in.PlaneSize != 0 {
		cStruct.plane_size = C.uint16_t(in.PlaneSize)
	}
	if in.Priority != 0 {
		cStruct.priority = C.uint32_t(in.Priority)
	}
	if in.Profile != 0 {
		cStruct.profile = C.uint32_t(in.Profile)
	}
	if in.Qos != "" {
		qosS := C.CString(in.Qos)
		defer C.free(unsafe.Pointer(qosS))
		cStruct.qos = qosS
	}
	if in.Reboot != 0 {
		cStruct.reboot = C.uint16_t(in.Reboot)
	}
	if in.RespHost != "" {
		respHostS := C.CString(in.RespHost)
		defer C.free(unsafe.Pointer(respHostS))
		cStruct.resp_host = respHostS
	}
	if in.ReqNodes != "" {
		reqNodesS := C.CString(in.ReqNodes)
		defer C.free(unsafe.Pointer(reqNodesS))
		cStruct.req_nodes = reqNodesS
	}
	if in.Requeue != 0 {
		cStruct.requeue = C.uint16_t(in.Requeue)
	}
	if in.Reservation != "" {
		reservationS := C.CString(in.Reservation)
		defer C.free(unsafe.Pointer(reservationS))
		cStruct.reservation = reservationS
	}
	if in.Script != "" {
		scriptS := C.CString(in.Script)
		defer C.free(unsafe.Pointer(scriptS))
		cStruct.script = scriptS
	}
	if in.Shared != 0 {
		cStruct.shared = C.uint16_t(in.Shared)
	}
	if in.SpankJobEnvSize != 0 {
		cStruct.spank_job_env_size = C.uint32_t(in.SpankJobEnvSize)
	}
	if in.TaskDist != 0 {
		cStruct.task_dist = C.uint16_t(in.TaskDist)
	}
	if in.TimeLimit != 0 {
		cStruct.time_limit = C.uint32_t(in.TimeLimit)
	}
	if in.TimeMin != 0 {
		cStruct.time_min = C.uint32_t(in.TimeMin)
	}
	//if go_struct.User_id != 0 {
	//	c_struct.user_id = C.uint32_t(go_struct.User_id)
	//}
	cStruct.user_id = C.uint32_t(in.UserId)

	if in.WaitAllNodes != 0 {
		cStruct.wait_all_nodes = C.uint16_t(in.WaitAllNodes)
	}
	if in.WarnSignal != 0 {
		cStruct.warn_signal = C.uint16_t(in.WarnSignal)
	}
	if in.WarnTime != 0 {
		cStruct.warn_time = C.uint16_t(in.WarnTime)
	}
	if in.WorkDir != "" {
		workDirS := C.CString(in.WorkDir)
		defer C.free(unsafe.Pointer(workDirS))
		cStruct.work_dir = workDirS
	}
	if in.CpusPerTask != 0 {
		cStruct.cpus_per_task = C.uint16_t(in.CpusPerTask)
	}
	if in.MinCpus != 0 {
		cStruct.min_cpus = C.uint32_t(in.MinCpus)
	}
	if in.MaxCpus != 0 {
		cStruct.max_cpus = C.uint32_t(in.MaxCpus)
	}
	if in.MinNodes != 0 {
		cStruct.min_nodes = C.uint32_t(in.MinNodes)
	}
	if in.MaxNodes != 0 {
		cStruct.max_nodes = C.uint32_t(in.MaxNodes)
	}
	if in.BoardsPerNode != 0 {
		cStruct.boards_per_node = C.uint16_t(in.BoardsPerNode)
	}
	if in.SocketsPerBoard != 0 {
		cStruct.sockets_per_board = C.uint16_t(in.SocketsPerBoard)
	}
	if in.SocketsPerNode != 0 {
		cStruct.sockets_per_node = C.uint16_t(in.SocketsPerNode)
	}
	if in.CoresPerSocket != 0 {
		cStruct.cores_per_socket = C.uint16_t(in.CoresPerSocket)
	}
	if in.ThreadsPerCore != 0 {
		cStruct.threads_per_core = C.uint16_t(in.ThreadsPerCore)
	}
	if in.NtasksPerNode != 0 {
		cStruct.ntasks_per_node = C.uint16_t(in.NtasksPerNode)
	}
	if in.NtasksPerSocket != 0 {
		cStruct.ntasks_per_socket = C.uint16_t(in.NtasksPerSocket)
	}
	if in.NtasksPerCore != 0 {
		cStruct.ntasks_per_core = C.uint16_t(in.NtasksPerCore)
	}
	if in.NtasksPerBoard != 0 {
		cStruct.ntasks_per_board = C.uint16_t(in.NtasksPerBoard)
	}
	if in.PnMinCpus != 0 {
		cStruct.pn_min_cpus = C.uint16_t(in.PnMinCpus)
	}
	if in.PnMinMemory != 0 {
		cStruct.pn_min_memory = C.uint32_t(in.PnMinMemory)
	}
	if in.PnMinTmpDisk != 0 {
		cStruct.pn_min_tmp_disk = C.uint32_t(in.PnMinTmpDisk)
	}
	if in.ReqSwitch != 0 {
		cStruct.req_switch = C.uint32_t(in.ReqSwitch)
	}
	if in.StdErr != "" {
		stdErrS := C.CString(in.StdErr)
		defer C.free(unsafe.Pointer(stdErrS))
		cStruct.std_err = stdErrS
	}
	if in.StdIn != "" {
		stdInS := C.CString(in.StdIn)
		defer C.free(unsafe.Pointer(stdInS))
		cStruct.std_in = stdInS
	}
	if in.StdOut != "" {
		stdOutS := C.CString(in.StdOut)
		defer C.free(unsafe.Pointer(stdOutS))
		cStruct.std_out = stdOutS
	}

	if in.Wait4Switch != 0 {
		cStruct.wait4switch = C.uint32_t(in.Wait4Switch)
	}
	if in.Wckey != "" {
		wckeyS := C.CString(in.Wckey)
		defer C.free(unsafe.Pointer(wckeyS))
		cStruct.wckey = wckeyS
	}

	cMsg := C.submit_job(&cStruct)

	defer C.free_submit_response_msg(cMsg)
	if cMsg == nil {
		goMsg := hpcTH.SubmitJobResp{}
		goMsg.SubmitResponseMsg[0].JobId = 1<<31 - 1
		goMsg.SubmitResponseMsg[0].ErrorCode = uint32(C.slurm_get_errno())
		return &goMsg, nil
	}
	goMsg := submitResponseMsgConvertCToGo(cMsg)

	return &goMsg, nil
}

func submitResponseMsgConvertCToGo(cStruct *C.struct_submit_response_msg) hpcTH.SubmitJobResp {
	var goStruct hpcTH.SubmitJobResp

	submitRespMsg := hpcTH.SubmitResponseMsg{}
	submitRespMsg.JobId = uint32(cStruct.job_id)
	submitRespMsg.StepId = uint32(cStruct.step_id)
	submitRespMsg.ErrorCode = uint32(cStruct.error_code)
	goStruct.SubmitResponseMsg = append(goStruct.SubmitResponseMsg, &submitRespMsg)
	return goStruct
}
