package logic

import (
	"PCM/adaptor/PCM-CORE/rpc/pcmcoreclient"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/internal/svc"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/kubenative"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/kubenativeclient"
	"PCM/common/tool"
	"context"
	"github.com/zeromicro/go-zero/core/logx"
	v1 "k8s.io/api/apps/v1"
	"time"
)

func InitCron(svc *svc.ServiceContext) {
	svc.Cron.AddFunc("*/5  *  *  *  *  ?", func() {
		// 查询core端分发下来的任务列表
		infoReq := pcmcoreclient.InfoListReq{
			Kind:        "cloud",
			ServiceName: "kubeNative",
		}
		infoList, err := svc.PcmCoreRpc.InfoList(context.Background(), &infoReq)
		if err != nil {
			logx.Error(err)
			return
		}
		// 提交任务
		applyYamlLogic := NewApplyYamlLogic(context.Background(), svc)
		for index, _ := range infoList.CloudInfoList {
			applyReq := kubenativeclient.ApplyReq{
				YamlString: infoList.CloudInfoList[index].YamlString,
			}
			infoList.CloudInfoList[index].Status = "Pending"
			_, err := applyYamlLogic.ApplyYaml(&applyReq)
			if err != nil {
				return
			}
		}
		// 查询Deployment列表
		listLogic := NewListLogic(context.Background(), svc)
		listReq := kubenative.ListReq{
			YamlString: "apiVersion: apps/v1\nkind: Deployment\n",
		}
		resp, err := listLogic.List(&listReq)
		var deploymentList v1.DeploymentList
		tool.K8sUnstructured(resp.Data, &deploymentList)
		if err != nil {
			return
		}

		// 遍历core端任务列表信息
		for index, _ := range infoList.CloudInfoList {
			for _, deployment := range deploymentList.Items {
				if deployment.Namespace == infoList.CloudInfoList[index].Namespace && deployment.Name == infoList.CloudInfoList[index].Name {
					infoList.CloudInfoList[index].StartTime = tool.TimeRemoveZone(deployment.Status.Conditions[0].LastTransitionTime.Time).String()
					infoList.CloudInfoList[index].RunningTime = time.Now().Sub(deployment.Status.Conditions[0].LastTransitionTime.Time).Milliseconds() / 1000
					// 判断状态
					if deployment.Status.ReadyReplicas == deployment.Status.Replicas {
						infoList.CloudInfoList[index].Status = "Running"
					} else {
						infoList.CloudInfoList[index].Status = "Pending"
					}
				}
			}
		}
		if len(infoList.CloudInfoList) != 0 {
			// 同步信息到core端
			SyncInfoReq := pcmcoreclient.SyncInfoReq{
				Kind:          "cloud",
				ServiceName:   "kubeNative",
				CloudInfoList: infoList.CloudInfoList,
			}
			svc.PcmCoreRpc.SyncInfo(context.Background(), &SyncInfoReq)
		}

	})
}
