package logic

import (
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/internal/svc"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/kubenative"
	"bytes"
	"context"
	"fmt"
	"github.com/sirupsen/logrus"
	"io"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
	"k8s.io/apimachinery/pkg/runtime"
	"k8s.io/apimachinery/pkg/util/yaml"
	"k8s.io/client-go/restmapper"

	"github.com/zeromicro/go-zero/core/logx"
)

type DeleteYamlLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewDeleteYamlLogic(ctx context.Context, svcCtx *svc.ServiceContext) *DeleteYamlLogic {
	return &DeleteYamlLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *DeleteYamlLogic) DeleteYaml(in *kubenative.ApplyReq) (*kubenative.Resp, error) {

	msg := &kubenative.Resp{
		Code: "200",
		Msg:  "success",
		Data: "",
	}
	d := yaml.NewYAMLOrJSONDecoder(bytes.NewBufferString(in.YamlString), 4096)

	dc := l.svcCtx.ClientSet.Discovery()
	restMapperRes, err := restmapper.GetAPIGroupResources(dc)
	if err != nil {
		logx.WithContext(context.TODO()).Errorf("delete %v   %v \n", err)
		return nil, fmt.Errorf("delete gvr is false %v", err)
	}

	restMapper := restmapper.NewDiscoveryRESTMapper(restMapperRes)

	for {
		ext := runtime.RawExtension{}

		if err := d.Decode(&ext); err != nil {
			if err == io.EOF {
				break
			}
			logx.WithContext(context.TODO()).Errorf("delete %v   %v \n", err)
		}

		obj, gvk, err := unstructured.UnstructuredJSONScheme.Decode(ext.Raw, nil, nil)
		if err != nil {
			logrus.Error(err)
			return nil, fmt.Errorf("delete gvr is false %v", err)
		}

		mapping, err := restMapper.RESTMapping(gvk.GroupKind(), gvk.Version)
		if err != nil {
			logrus.Error(err)
			return nil, fmt.Errorf("delete gvr is false %v", err)
		}

		// runtime.Object转换为unstructed
		unstructuredObj, err := runtime.DefaultUnstructuredConverter.ToUnstructured(obj)
		if err != nil {
			logrus.Error(err)
			return nil, fmt.Errorf("delete gvr is false %v", err)
		}

		var unstruct unstructured.Unstructured

		unstruct.Object = unstructuredObj

		tmpMetadata := unstructuredObj["metadata"].(map[string]interface{})
		tmpName := tmpMetadata["name"].(string)
		tmpKind := unstructuredObj["kind"].(string)

		if unstruct.GetNamespace() == "" {
			unstruct.SetNamespace("default")
		}
		logrus.Info("deleting resource name: " + tmpName + ", kind: " + tmpKind + ", ns: " + unstruct.GetNamespace())

		if unstruct.GetNamespace() == "" {
			err := l.svcCtx.DynamicClient.Resource(mapping.Resource).Delete(context.TODO(), tmpName, metav1.DeleteOptions{})
			if err != nil {
				logrus.Error(err)
				return nil, fmt.Errorf("delete gvr is false %v", err)
			}
		} else {
			err := l.svcCtx.DynamicClient.Resource(mapping.Resource).Namespace(unstruct.GetNamespace()).Delete(context.TODO(), tmpName, metav1.DeleteOptions{})
			if err != nil {
				logrus.Error(err)
				return nil, fmt.Errorf("delete gvr is false %v", err)
			}
		}

	}
	if err != nil {
		msg.Code = "500"
		msg.Msg = "failed"
		msg.Data = "服务繁忙"
		return msg, nil
	}
	return msg, nil
}
