package logic

import (
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/internal/svc"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/kubenative"
	"PCM/common/tool"
	"bytes"
	"context"
	"encoding/json"
	"fmt"
	"io"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
	"k8s.io/apimachinery/pkg/runtime"
	syaml "k8s.io/apimachinery/pkg/runtime/serializer/yaml"
	"k8s.io/apimachinery/pkg/util/yaml"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetLogic {
	return &GetLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetLogic) Get(in *kubenative.GetReq) (*kubenative.Resp, error) {
	msg := &kubenative.Resp{
		Code: "200",
		Msg:  "success",
		Data: "",
	}
	var data *unstructured.Unstructured
	d := yaml.NewYAMLOrJSONDecoder(bytes.NewBufferString(in.YamlString), 4096)
	var err error
	for {
		var (
			rawObj runtime.RawExtension
		)
		err := d.Decode(&rawObj)
		if err == io.EOF {
			break
		}

		obj, _, err := syaml.NewDecodingSerializer(unstructured.UnstructuredJSONScheme).Decode(rawObj.Raw, nil, nil)

		if err != nil {
			return nil, fmt.Errorf("rawobj is err%v", err)
		}

		unstructuredMap, err := runtime.DefaultUnstructuredConverter.ToUnstructured(obj)
		if err != nil {
			return nil, fmt.Errorf("tounstructured is err %v", err)
		}

		unstructuredObj := &unstructured.Unstructured{Object: unstructuredMap}
		gvr, err := tool.GetGVR(l.svcCtx.ClientSet, unstructuredObj.GroupVersionKind())
		if err != nil {
			return nil, fmt.Errorf("get gvr is false %v", err)
		}

		if unstructuredObj.GetNamespace() == "" {
			unstructuredObj.SetNamespace("default")
		}

		data, err = l.svcCtx.DynamicClient.Resource(gvr).Namespace(unstructuredObj.GetNamespace()).Get(context.TODO(), unstructuredObj.GetName(), metav1.GetOptions{})
		if err != nil {
			return nil, fmt.Errorf("get resource err %v", err)
		}
	}

	if err != nil && err.Error() != "EOF" {
		marshal, _ := json.Marshal(err)
		msg.Code = "500"
		msg.Msg = "failed"
		msg.Data = string(marshal)
		return msg, nil
	}
	if data != nil {
		marshal, _ := json.Marshal(data)
		msg.Data = string(marshal)
	}
	return msg, nil
}
