package svc

import (
	"PCM/adaptor/PCM-CORE/rpc/pcmcoreclient"
	"PCM/adaptor/PCM-K8S/PCM-K8S-NATIVE/rpc/internal/config"
	"github.com/robfig/cron/v3"
	"github.com/zeromicro/go-zero/zrpc"
	"k8s.io/client-go/dynamic"
	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/rest"
	"k8s.io/client-go/util/flowcontrol"
)

type ServiceContext struct {
	Config        config.Config
	DynamicClient dynamic.Interface
	ClientSet     *kubernetes.Clientset
	Cron          *cron.Cron
	PcmCoreRpc    pcmcoreclient.PcmCore
}

func NewServiceContext(c config.Config) *ServiceContext {

	restConfig := &rest.Config{
		Host:        c.KubeConfig.Server,
		RateLimiter: flowcontrol.NewTokenBucketRateLimiter(1000, 1000),
		BearerToken: c.KubeConfig.Token,
		TLSClientConfig: rest.TLSClientConfig{
			Insecure: true,
		},
	}
	dynamicClient, _ := dynamic.NewForConfig(restConfig)
	clientSet, _ := kubernetes.NewForConfig(restConfig)

	return &ServiceContext{
		Cron:          cron.New(cron.WithSeconds()),
		Config:        c,
		DynamicClient: dynamicClient,
		ClientSet:     clientSet,
		PcmCoreRpc:    pcmcoreclient.NewPcmCore(zrpc.MustNewClient(c.PcmCoreRpcConf)),
	}
}
