package main

import (
	"PCM/adaptor/PCM-STORAGE/PCM-CEPH/rpc/ceph"
	"PCM/adaptor/PCM-STORAGE/PCM-CEPH/rpc/internal/config"
	"PCM/adaptor/PCM-STORAGE/PCM-CEPH/rpc/internal/server"
	"PCM/adaptor/PCM-STORAGE/PCM-CEPH/rpc/internal/svc"
	commonConfig "PCM/common/config"
	"PCM/common/interceptor/rpcserver"
	"flag"
	"github.com/zeromicro/go-zero/core/conf"
	"github.com/zeromicro/go-zero/core/logx"
	"github.com/zeromicro/go-zero/core/service"
	"github.com/zeromicro/go-zero/zrpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/reflection"
)

var configFile = flag.String("f", "adaptor/PCM-STORAGE/PCM-CEPH/rpc/etc/pcmceph.yaml", "the config file")

func main() {
	flag.Parse()

	var bootstrapConfig commonConfig.BootstrapConfig
	//conf.MustLoad(*configFile, &c)
	conf.MustLoad(*configFile, &bootstrapConfig)

	//解析业务配置
	var c config.Config
	nacosConfig := bootstrapConfig.NacosConfig

	serviceConfigContent := nacosConfig.InitConfig(func(data string) {
		err := conf.LoadFromYamlBytes([]byte(data), &c)
		if err != nil {
			panic(err)
		}
	})
	err := conf.LoadFromYamlBytes([]byte(serviceConfigContent), &c)
	if err != nil {
		panic(err)
	}
	// start log component
	logx.MustSetup(c.LogConf)
	// 注册到nacos
	nacosConfig.Discovery(&c.RpcServerConf)

	ctx := svc.NewServiceContext(c)
	//ctx.Cron.Start()

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		ceph.RegisterCephServer(grpcServer, server.NewCephServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})

	//rpc log
	s.AddUnaryInterceptors(rpcserver.LoggerInterceptor)

	defer s.Stop()
	// 初始化定时任务
	//logic.InitCron(ctx)

	logx.Infof("Starting rpc server at %s...\n", c.ListenOn)

	s.Start()

	/*ctx := svc.NewServiceContext(c)

	s := zrpc.MustNewServer(c.RpcServerConf, func(grpcServer *grpc.Server) {
		ceph.RegisterCephServer(grpcServer, server.NewCephServer(ctx))

		if c.Mode == service.DevMode || c.Mode == service.TestMode {
			reflection.Register(grpcServer)
		}
	})
	defer s.Stop()

	fmt.Printf("Starting rpc server at %s...\n", c.ListenOn)
	s.Start()*/
}
